@@ -13,16 +13,17 @@
 #    under the License.
 
 import logging
+import queue
 import time
 
 from ovs.db import idl
-from six.moves import queue as Queue
 
 from ovsdbapp import api
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import exceptions
 
 LOG = logging.getLogger(__name__)
+MAX_SLEEP = 8
 
 
 class Transaction(api.Transaction):
@@ -32,7 +33,7 @@ class Transaction(api.Transaction):
         self.check_error = check_error
         self.log_errors = log_errors
         self.commands = []
-        self.results = Queue.Queue(1)
+        self.results = queue.Queue(1)
         self.ovsdb_connection = ovsdb_connection
         self.timeout = timeout or ovsdb_connection.timeout
 
@@ -52,9 +53,11 @@ class Transaction(api.Transaction):
         self.ovsdb_connection.queue_txn(self)
         try:
             result = self.results.get(timeout=self.timeout)
-        except Queue.Empty:
-            raise exceptions.TimeoutException(commands=self.commands,
-                                              timeout=self.timeout)
+        except queue.Empty as e:
+            raise exceptions.TimeoutException(
+                commands=self.commands,
+                timeout=self.timeout,
+                cause='Result queue is empty') from e
         if isinstance(result, idlutils.ExceptionResult):
             if self.log_errors:
                 LOG.error(result.tb)
@@ -72,6 +75,7 @@ class Transaction(api.Transaction):
     def do_commit(self):
         self.start_time = time.time()
         attempts = 0
+        retries = 0
         if not self.commands:
             LOG.debug("There are no commands to commit")
             return []
@@ -101,8 +105,14 @@ class Transaction(api.Transaction):
                 # TRY_AGAIN until we time out and Connection.run() calls
                 # idl.run() again. So, call idl.run() here just in case.
                 self.api.idl.run()
+
+                # In the event that there is an issue with the txn or the db
+                # is down, don't spam new txns as fast as we can
+                time.sleep(min(2 ** retries, self.time_remaining(), MAX_SLEEP))
+                retries += 1
                 continue
-            elif status in (txn.ERROR, txn.NOT_LOCKED):
+            retries = 0
+            if status in (txn.ERROR, txn.NOT_LOCKED):
                 msg = 'OVSDB Error: '
                 if status == txn.NOT_LOCKED:
                     msg += ("The transaction failed because the IDL has "
@@ -117,10 +127,10 @@ class Transaction(api.Transaction):
                     # For now, raise similar error to vsctl/utils.execute()
                     raise RuntimeError(msg)
                 return
-            elif status == txn.ABORTED:
+            if status == txn.ABORTED:
                 LOG.debug("Transaction aborted")
                 return
-            elif status == txn.UNCHANGED:
+            if status == txn.UNCHANGED:
                 LOG.debug("Transaction caused no change")
             elif status == txn.SUCCESS:
                 self.post_commit(txn)
