@@ -13,17 +13,14 @@
 import abc
 import atexit
 import logging
+import queue
 import threading
 
-import six
-from six.moves import queue as Queue
-
 LOG = logging.getLogger(__name__)
 STOP_EVENT = ("STOP", None, None, None)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class RowEvent(object):
+class RowEvent(object, metaclass=abc.ABCMeta):
     ROW_CREATE = "create"
     ROW_UPDATE = "update"
     ROW_DELETE = "delete"
@@ -81,7 +78,7 @@ class WaitEvent(RowEvent):
     def __init__(self, *args, **kwargs):
         self.event = threading.Event()
         self.timeout = kwargs.pop('timeout', None)
-        super(WaitEvent, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @abc.abstractmethod
     def matches(self, event, row, old=None):
@@ -96,9 +93,9 @@ class WaitEvent(RowEvent):
 
 class RowEventHandler(object):
     def __init__(self):
-        self.__watched_events = set()
-        self.__lock = threading.Lock()
-        self.notifications = Queue.Queue()
+        self._watched_events = set()
+        self._lock = threading.Lock()
+        self.notifications = queue.Queue()
         self.notify_thread = threading.Thread(target=self.notify_loop)
         self.notify_thread.daemon = True
         atexit.register(self.shutdown)
@@ -107,28 +104,36 @@ class RowEventHandler(object):
     def start(self):
         self.notify_thread.start()
 
+    @staticmethod
+    def match(candidate, event, row, updates):
+        try:
+            return candidate.matches(event, row, updates)
+        except Exception:
+            LOG.exception("Event not matched due to this exception:\n")
+            return False
+
     def matching_events(self, event, row, updates):
-        with self.__lock:
-            return tuple(t for t in self.__watched_events
-                         if t.matches(event, row, updates))
+        with self._lock:
+            return tuple(t for t in self._watched_events
+                         if self.match(t, event, row, updates))
 
     def watch_event(self, event):
-        with self.__lock:
-            self.__watched_events.add(event)
+        with self._lock:
+            self._watched_events.add(event)
 
     def watch_events(self, events):
-        with self.__lock:
+        with self._lock:
             for event in events:
-                self.__watched_events.add(event)
+                self._watched_events.add(event)
 
     def unwatch_event(self, event):
-        with self.__lock:
-            self.__watched_events.discard(event)
+        with self._lock:
+            self._watched_events.discard(event)
 
     def unwatch_events(self, events):
-        with self.__lock:
+        with self._lock:
             for event in events:
-                self.__watched_events.discard(event)
+                self._watched_events.discard(event)
 
     def shutdown(self):
         self.notifications.put(STOP_EVENT)
@@ -150,6 +155,17 @@ class RowEventHandler(object):
                 LOG.exception('Unexpected exception in notify_loop')
 
     def notify(self, event, row, updates=None):
+        """Method for calling backend to call for each DB update
+
+        :param event:   Backend representation of event type, e.g.
+                        create, update, delete
+        :param row:     Backend representation of a Row object. If it is not
+                        immutable, it should be converted or guaranteed not to
+                        be changed in other threads.
+        :param updates: Backend representation of updates to a Row. e.g.
+                        a Row object with just changed attributes, a
+                        dictionary of changes, etc.
+        """
         matching = self.matching_events(
             event, row, updates)
         for match in matching:
