@@ -26,7 +26,7 @@ class AddManagerCommand(command.AddCommand):
     table_name = 'Manager'
 
     def __init__(self, api, target):
-        super(AddManagerCommand, self).__init__(api)
+        super().__init__(api)
         self.target = target
 
     def run_idl(self, txn):
@@ -41,9 +41,9 @@ class AddManagerCommand(command.AddCommand):
         self.result = row.uuid
 
 
-class GetManagerCommand(BaseCommand):
+class GetManagerCommand(command.ReadOnlyCommand):
     def __init__(self, api):
-        super(GetManagerCommand, self).__init__(api)
+        super().__init__(api)
 
     def run_idl(self, txn):
         self.result = [m.target for m in
@@ -52,17 +52,17 @@ class GetManagerCommand(BaseCommand):
 
 class RemoveManagerCommand(BaseCommand):
     def __init__(self, api, target):
-        super(RemoveManagerCommand, self).__init__(api)
+        super().__init__(api)
         self.target = target
 
     def run_idl(self, txn):
         try:
             manager = idlutils.row_by_value(self.api.idl, 'Manager', 'target',
                                             self.target)
-        except idlutils.RowNotFound:
+        except idlutils.RowNotFound as e:
             msg = "Manager with target %s does not exist" % self.target
             LOG.error(msg)
-            raise RuntimeError(msg)
+            raise RuntimeError(msg) from e
         try:
             self.api._ovs.delvalue('manager_options', manager)
         except AttributeError:  # OVS < 2.6
@@ -77,7 +77,7 @@ class AddBridgeCommand(command.AddCommand):
     table_name = 'Bridge'
 
     def __init__(self, api, name, may_exist, datapath_type):
-        super(AddBridgeCommand, self).__init__(api)
+        super().__init__(api)
         self.name = name
         self.may_exist = may_exist
         self.datapath_type = datapath_type
@@ -113,7 +113,7 @@ class AddBridgeCommand(command.AddCommand):
 
 class DelBridgeCommand(BaseCommand):
     def __init__(self, api, name, if_exists):
-        super(DelBridgeCommand, self).__init__(api)
+        super().__init__(api)
         self.name = name
         self.if_exists = if_exists
 
@@ -121,13 +121,13 @@ class DelBridgeCommand(BaseCommand):
         try:
             br = idlutils.row_by_value(self.api.idl, 'Bridge', 'name',
                                        self.name)
-        except idlutils.RowNotFound:
+        except idlutils.RowNotFound as e:
             if self.if_exists:
                 return
             else:
                 msg = "Bridge %s does not exist" % self.name
                 LOG.error(msg)
-                raise RuntimeError(msg)
+                raise RuntimeError(msg) from e
         # Clean up cached ports/interfaces
         for port in br.ports:
             for interface in port.interfaces:
@@ -143,9 +143,9 @@ class DelBridgeCommand(BaseCommand):
         br.delete()
 
 
-class BridgeExistsCommand(BaseCommand):
+class BridgeExistsCommand(command.ReadOnlyCommand):
     def __init__(self, api, name):
-        super(BridgeExistsCommand, self).__init__(api)
+        super().__init__(api)
         self.name = name
 
     def run_idl(self, txn):
@@ -153,9 +153,9 @@ class BridgeExistsCommand(BaseCommand):
                                                  'name', self.name, None))
 
 
-class ListBridgesCommand(BaseCommand):
+class ListBridgesCommand(command.ReadOnlyCommand):
     def __init__(self, api):
-        super(ListBridgesCommand, self).__init__(api)
+        super().__init__(api)
 
     def run_idl(self, txn):
         # NOTE (twilson) [x.name for x in rows.values()] if no index
@@ -163,34 +163,9 @@ class ListBridgesCommand(BaseCommand):
                        self.api._tables['Bridge'].rows.values()]
 
 
-class BrGetExternalIdCommand(BaseCommand):
-    def __init__(self, api, name, field):
-        super(BrGetExternalIdCommand, self).__init__(api)
-        self.name = name
-        self.field = field
-
-    def run_idl(self, txn):
-        br = idlutils.row_by_value(self.api.idl, 'Bridge', 'name', self.name)
-        self.result = br.external_ids[self.field]
-
-
-class BrSetExternalIdCommand(BaseCommand):
-    def __init__(self, api, name, field, value):
-        super(BrSetExternalIdCommand, self).__init__(api)
-        self.name = name
-        self.field = field
-        self.value = value
-
-    def run_idl(self, txn):
-        br = idlutils.row_by_value(self.api.idl, 'Bridge', 'name', self.name)
-        external_ids = getattr(br, 'external_ids', {})
-        external_ids[self.field] = self.value
-        br.external_ids = external_ids
-
-
 class SetControllerCommand(BaseCommand):
     def __init__(self, api, bridge, targets):
-        super(SetControllerCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
         self.targets = targets
 
@@ -207,7 +182,7 @@ class SetControllerCommand(BaseCommand):
 
 class DelControllerCommand(BaseCommand):
     def __init__(self, api, bridge):
-        super(DelControllerCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
 
     def run_idl(self, txn):
@@ -215,9 +190,9 @@ class DelControllerCommand(BaseCommand):
         br.controller = []
 
 
-class GetControllerCommand(BaseCommand):
+class GetControllerCommand(command.ReadOnlyCommand):
     def __init__(self, api, bridge):
-        super(GetControllerCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
 
     def run_idl(self, txn):
@@ -227,7 +202,7 @@ class GetControllerCommand(BaseCommand):
 
 class SetFailModeCommand(BaseCommand):
     def __init__(self, api, bridge, mode):
-        super(SetFailModeCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
         self.mode = mode
 
@@ -240,7 +215,7 @@ class AddPortCommand(command.AddCommand):
     table_name = 'Port'
 
     def __init__(self, api, bridge, port, may_exist):
-        super(AddPortCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
         self.port = port
         self.may_exist = may_exist
@@ -273,7 +248,7 @@ class AddPortCommand(command.AddCommand):
 
 class DelPortCommand(BaseCommand):
     def __init__(self, api, port, bridge, if_exists):
-        super(DelPortCommand, self).__init__(api)
+        super().__init__(api)
         self.port = port
         self.bridge = bridge
         self.if_exists = if_exists
@@ -282,11 +257,11 @@ class DelPortCommand(BaseCommand):
         try:
             port = idlutils.row_by_value(self.api.idl, 'Port', 'name',
                                          self.port)
-        except idlutils.RowNotFound:
+        except idlutils.RowNotFound as e:
             if self.if_exists:
                 return
             msg = "Port %s does not exist" % self.port
-            raise RuntimeError(msg)
+            raise RuntimeError(msg) from e
         if self.bridge:
             br = idlutils.row_by_value(self.api.idl, 'Bridge', 'name',
                                        self.bridge)
@@ -316,9 +291,9 @@ class DelPortCommand(BaseCommand):
         port.delete()
 
 
-class ListPortsCommand(BaseCommand):
+class ListPortsCommand(command.ReadOnlyCommand):
     def __init__(self, api, bridge):
-        super(ListPortsCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
 
     def run_idl(self, txn):
@@ -326,9 +301,9 @@ class ListPortsCommand(BaseCommand):
         self.result = [p.name for p in br.ports if p.name != self.bridge]
 
 
-class ListIfacesCommand(BaseCommand):
+class ListIfacesCommand(command.ReadOnlyCommand):
     def __init__(self, api, bridge):
-        super(ListIfacesCommand, self).__init__(api)
+        super().__init__(api)
         self.bridge = bridge
 
     def run_idl(self, txn):
@@ -337,9 +312,9 @@ class ListIfacesCommand(BaseCommand):
                        for i in p.interfaces]
 
 
-class PortToBridgeCommand(BaseCommand):
+class PortToBridgeCommand(command.ReadOnlyCommand):
     def __init__(self, api, name):
-        super(PortToBridgeCommand, self).__init__(api)
+        super().__init__(api)
         self.name = name
 
     def run_idl(self, txn):
@@ -353,9 +328,9 @@ class PortToBridgeCommand(BaseCommand):
         self.result = next(br.name for br in bridges if port in br.ports)
 
 
-class InterfaceToBridgeCommand(BaseCommand):
+class InterfaceToBridgeCommand(command.ReadOnlyCommand):
     def __init__(self, api, name):
-        super(InterfaceToBridgeCommand, self).__init__(api)
+        super().__init__(api)
         self.name = name
 
     def run_idl(self, txn):
@@ -367,3 +342,56 @@ class InterfaceToBridgeCommand(BaseCommand):
 
         bridges = self.api._tables['Bridge'].rows.values()
         self.result = next(br.name for br in bridges if pname in br.ports)
+
+
+class GetExternalIdCommand(command.ReadOnlyCommand):
+    def __init__(self, api, table, name, field):
+        super().__init__(api)
+        self.table = table
+        self.name = name
+        self.field = field
+
+    def run_idl(self, txn):
+        row = idlutils.row_by_value(
+            self.api.idl, self.table, 'name', self.name)
+        self.result = row.external_ids[self.field]
+
+
+class SetExternalIdCommand(BaseCommand):
+    def __init__(self, api, table, name, field, value):
+        super().__init__(api)
+        self.table = table
+        self.name = name
+        self.field = field
+        self.value = value
+
+    def run_idl(self, txn):
+        row = idlutils.row_by_value(
+            self.api.idl, self.table, 'name', self.name)
+        external_ids = getattr(row, 'external_ids', {})
+        external_ids[self.field] = self.value
+        row.external_ids = external_ids
+
+
+class BrGetExternalIdCommand(GetExternalIdCommand):
+    def __init__(self, api, name, field):
+        super().__init__(
+            api, 'Bridge', name, field)
+
+
+class BrSetExternalIdCommand(SetExternalIdCommand):
+    def __init__(self, api, name, field, value):
+        super().__init__(
+            api, 'Bridge', name, field, value)
+
+
+class IfaceGetExternalIdCommand(GetExternalIdCommand):
+    def __init__(self, api, name, field):
+        super().__init__(
+            api, 'Interface', name, field)
+
+
+class IfaceSetExternalIdCommand(SetExternalIdCommand):
+    def __init__(self, api, name, field, value):
+        super().__init__(
+            api, 'Interface', name, field, value)
